# Integrating a Toolbar and Touch Bar into Your App

Build a toolbar in your app's window, with a corresponding Touch Bar, for quick access to app features.

## Overview

A [`toolbar`](https://developer.apple.com/documentation/appkit/views_and_controls/toolbar) manages the space immediately below a window's title bar and above your app's custom content. Toolbars manage controls that apply to the content into a window. 

Each item in the toolbar is represented by an [`NSToolbarItem`](https://developer.apple.com/documentation/appkit/nstoolbaritem) object, created by Interface Builder or in code. This sample shows you how to integrate a toolbar into an `NSWindow`.

## Create the Toolbar

Add a toolbar to your window by dragging an `NSToolbar` object from Xcode's Library window onto your `NSWindow` object. Then connect the toolbar object to the window controller and make it the [`NSToolbarDelegate`](https://developer.apple.com/documentation/appkit/nstoolbardelegate).

## Assign Toolbar Identifiers

For each `NSToolbarItem` you create, you must assign a unique identifier of type [`NSToolbarItem.Identifier`](https://developer.apple.com/documentation/appkit/nstoolbaritem/identifier). AppKit has predefined, built-in toolbar item identifiers for cloud sharing, printing, and showing the font and color palette. All toolbar items are  arranged within the toolbar in an order determined by the user, and the order is autosaved as part of the toolbar's configuration. 

This sample creates two identifiers: one for setting the font size and one for setting the font style of an `NSTextView`.

``` swift
private extension NSToolbarItem.Identifier {
    static let fontSize: NSToolbarItem.Identifier = NSToolbarItem.Identifier(rawValue: "FontSize")
    static let fontStyle: NSToolbarItem.Identifier = NSToolbarItem.Identifier(rawValue: "FontStyle")
}
```
[View in Source](x-source-tag://ItemIdentifiers)

## Create the Allowed Toolbar Items

Implement [`toolbarAllowedItemIdentifiers(_:)`](https://developer.apple.com/documentation/appkit/nstoolbardelegate/1516995-toolbaralloweditemidentifiers) as a required function to return an array of toolbar item identifiers for the toolbar, specifying the contents and the order of the items in the configuration palette.

``` swift
func toolbarAllowedItemIdentifiers(_ toolbar: NSToolbar) -> [NSToolbarItem.Identifier] {
    return [ NSToolbarItem.Identifier.fontStyle,
             NSToolbarItem.Identifier.fontSize,
             NSToolbarItem.Identifier.space,
             NSToolbarItem.Identifier.flexibleSpace,
             NSToolbarItem.Identifier.print ]
}
```
[View in Source](x-source-tag://AllowedToolbarItems)

## Create the Default Toolbar Items

Implement [`toolbarDefaultItemIdentifiers(_:)`](https://developer.apple.com/documentation/appkit/nstoolbardelegate/1516944-toolbardefaultitemidentifiers) to provide the default set of items that initially appear in the toolbar.

``` swift
func toolbarDefaultItemIdentifiers(_ toolbar: NSToolbar) -> [NSToolbarItem.Identifier] {
    return [.fontStyle, .fontSize]
}
```
[View in Source](x-source-tag://DefaultIdentifiers)

## Create a Toolbar Item from an Identifier

As the toolbar delegate, you are responsible for creating a toolbar item for each identifier you define. At the time that AppKit instructs you to create an `NSToolbarItem`, you create one based on the identifier. Implement [`toolbar(_:itemForItemIdentifier:willBeInsertedIntoToolbar:)`](https://developer.apple.com/documentation/appkit/nstoolbardelegate/1516985-toolbar) to return the two toolbar items: Font Style and Font Size.

``` swift
func toolbar(
    _ toolbar: NSToolbar,
    itemForItemIdentifier itemIdentifier: NSToolbarItem.Identifier,
    willBeInsertedIntoToolbar flag: Bool) -> NSToolbarItem? {
    
    var toolbarItem: NSToolbarItem = NSToolbarItem()
    
    /**	Create a new NSToolbarItem, and then go through the process of setting up its
    	attributes from the master toolbar item matching that identifier in the dictionary of items.
     */
    if itemIdentifier == NSToolbarItem.Identifier.fontStyle {
        // 1) Font style toolbar item.
        toolbarItem =
            customToolbarItem(itemForItemIdentifier: NSToolbarItem.Identifier.fontStyle.rawValue,
                              label: NSLocalizedString("Font Style", comment: ""),
                              paletteLabel: NSLocalizedString("Font Style", comment: ""),
                              toolTip: NSLocalizedString("tool tip font style", comment: ""),
                              itemContent: styleSegmentView)!
    } else if itemIdentifier == NSToolbarItem.Identifier.fontSize {
        // 2) Font size toolbar item.
        toolbarItem =
            customToolbarItem(itemForItemIdentifier: NSToolbarItem.Identifier.fontSize.rawValue,
                              label: NSLocalizedString("Font Size", comment: ""),
                              paletteLabel: NSLocalizedString("Font Size", comment: ""),
                              toolTip: NSLocalizedString("tool tip font size", comment: ""),
                              itemContent: fontSizeView)!
    }
    
    return toolbarItem
}
```
[View in Source](x-source-tag://ToolbarItemForIdentifier)

Each toolbar receives its own distinct set of identifiers. If a toolbar item has a custom view, that view should be in place when the item is returned. Do not assume that the returned item is added as an active toolbar item. In fact, the toolbar may ask for items here to construct the customization palette (it makes copies of the returned items). If `willBeInsertedIntoToolbar` is `true`, the returned item is inserted, and eventually the `toolbarWillAddItem:` delegate is called.

## Use a Custom View for a Toolbar Item

When you apply a custom view to an `NSToolbarItem`, you place one or more controls together to represent that `NSToolbarItem`. In this sample, the custom views included are the font-style segmented control and the font size control, consisting of a label, stepper, and edit field.

## Add More Attributes to a Toolbar Item

Use the optional [`toolbarWillAddItem(_:)`](https://developer.apple.com/documentation/appkit/nstoolbardelegate/1516964-toolbarwilladditem) delegate function. This function is called when a new item is about to be added to the toolbar. At this time, you add or change initial state information for toolbar items, particularly those that you don't directly control, like `NSToolbarPrintItemIdentifier`. In this sample, you add a tooltip string to the print toolbar item.

## Customize the Toolbar

You implement the delegate function `toolbarAllowedItemIdentifiers` to determine which toolbar items users place in the toolbar. Users customize the item ordering by either control-clicking the toolbar, or by choosing Customize Toolbar from the View menu. This option opens a customization alert, where the user changes the layout. Only items returned by the delegate function `toolbarAllowedItemIdentifiers` appear in that alert. 

## Add Touch Bar Support

This sample implements the [`Touch Bar`](https://developer.apple.com/documentation/appkit/touch_bar) to work in conjunction with the toolbar. What you see in the window's toolbar is what you see in the Touch Bar. This sample uses the Touch Bar to set the font style and size the same way as the toolbar.

Override `makeTouchBar()` and create the [`NSTouchBar`](https://developer.apple.com/documentation/appkit/nstouchbar).

``` swift
override func makeTouchBar() -> NSTouchBar? {
    let touchBar = NSTouchBar()
    touchBar.delegate = self
    touchBar.customizationIdentifier = .touchBar
    touchBar.defaultItemIdentifiers = [.fontStyle, .popover, NSTouchBarItem.Identifier.otherItemsProxy]
    touchBar.customizationAllowedItemIdentifiers = [.fontStyle, .popover]
    
    return touchBar
}
```
[View in Source](x-source-tag://CreateTouchBar)

Test the Touch Bar on a MacBook Pro. If you're unable to do so, however, you may use Xcode's Touch Bar simulator by choosing Window > Show Touch Bar.

## Customize the Touch Bar

The user customizes the Touch Bar by choosing Customize Touch Bar from the View menu. To add this menu item to the main menu, implement the following:

``` swift
NSApplication.shared.isAutomaticCustomizeTouchBarMenuItemEnabled = true
```












